@@ -58,26 +58,19 @@ namespace Banshee.Sources
         protected DatabaseTrackListModel track_model;
         protected DatabaseAlbumListModel album_model;
         protected DatabaseArtistListModel artist_model;
-        
-        private DatabaseQueryFilterModel<string> genre_model;
-        protected DatabaseQueryFilterModel<string> GenreModel {
-            get {
-                return genre_model ?? 
-                    genre_model = new Banshee.Collection.Database.DatabaseQueryFilterModel<string> (this, DatabaseTrackModel, ServiceManager.DbConnection,
-                        Catalog.GetString ("All Genres ({0})"), UniqueId, BansheeQuery.GenreField, "Genre");
-            }
-        }
+        protected DatabaseQueryFilterModel<string> genre_model;
 
-        protected RateLimiter reload_limiter;
-        
-        protected string type_unique_id;
-        protected override string TypeUniqueId {
-            get { return type_unique_id; }
+        private RateLimiter reload_limiter;
+
+        public DatabaseSource (string generic_name, string name, string id, int order) : this (generic_name, name, id, order, null)
+        {
         }
         
-        public DatabaseSource (string generic_name, string name, string id, int order) : base (generic_name, name, order)
+        public DatabaseSource (string generic_name, string name, string id, int order, Source parent) : base (generic_name, name, order, id)
         {
-            type_unique_id = id;
+            if (parent != null) {
+                SetParentSource (parent);
+            }
             DatabaseSourceInitialize ();
         }
 
@@ -85,6 +78,13 @@ namespace Banshee.Sources
         {
         }
 
+        public void UpdateCounts ()
+        {
+            DatabaseTrackModel.UpdateUnfilteredAggregates ();
+            ever_counted = true;
+            OnUpdated ();
+        }
+
         public abstract void Save ();
 
         protected override void Initialize ()
@@ -97,11 +97,9 @@ namespace Banshee.Sources
             get { return true; }
         }
 
-        protected DatabaseTrackListModel DatabaseTrackModel {
-            get {
-                return track_model ?? track_model = new DatabaseTrackListModel (ServiceManager.DbConnection, TrackProvider, this);
-            }
-            set { track_model = value; }
+        public DatabaseTrackListModel DatabaseTrackModel {
+            get { return track_model ?? track_model = (Parent as DatabaseSource ?? this).CreateTrackModelFor (this); }
+            protected set { track_model = value; }
         }
 
         private IDatabaseTrackModelCache track_cache;
@@ -123,22 +121,48 @@ namespace Banshee.Sources
             
             current_filters_schema = CreateSchema<string[]> ("current_filters");
 
-            if (HasArtistAlbum) {
-                artist_model = new DatabaseArtistListModel (this, DatabaseTrackModel, ServiceManager.DbConnection, UniqueId);
-                album_model = new DatabaseAlbumListModel (this, DatabaseTrackModel, ServiceManager.DbConnection, UniqueId);
-                
-                AvailableFilters.Add (GenreModel);
-                AvailableFilters.Add (artist_model);
-                AvailableFilters.Add (album_model);
-                
-                DefaultFilters.Add (GenreModel);
-                DefaultFilters.Add (artist_model);
-                DefaultFilters.Add (album_model);
+            DatabaseSource filter_src = Parent as DatabaseSource ?? this;
+            foreach (IFilterListModel filter in filter_src.CreateFiltersFor (this)) {
+                AvailableFilters.Add (filter);
+                DefaultFilters.Add (filter);
             }
 
             reload_limiter = new RateLimiter (RateLimitedReload);
         }
 
+        protected virtual DatabaseTrackListModel CreateTrackModelFor (DatabaseSource src)
+        {
+            return new DatabaseTrackListModel (ServiceManager.DbConnection, TrackProvider, src);
+        }
+
+        protected virtual IEnumerable<IFilterListModel> CreateFiltersFor (DatabaseSource src)
+        {
+            if (!HasArtistAlbum) {
+                yield break;
+            }
+
+            DatabaseArtistListModel artist_model = new DatabaseArtistListModel (src, src.DatabaseTrackModel, ServiceManager.DbConnection, src.UniqueId);
+            DatabaseAlbumListModel album_model = new DatabaseAlbumListModel (src, src.DatabaseTrackModel, ServiceManager.DbConnection, src.UniqueId);
+            DatabaseQueryFilterModel<string> genre_model = new DatabaseQueryFilterModel<string> (src, src.DatabaseTrackModel, ServiceManager.DbConnection,
+                        Catalog.GetString ("All Genres ({0})"), src.UniqueId, BansheeQuery.GenreField, "Genre");
+
+            if (this == src) {
+                this.artist_model = artist_model;
+                this.album_model = album_model;
+                this.genre_model = genre_model;
+            }
+
+            yield return artist_model;
+            yield return album_model;
+            yield return genre_model;
+        }
+
+        protected virtual void AfterInitialized ()
+        {
+            DatabaseTrackModel.Initialize (TrackCache);
+            OnSetupComplete ();
+        }
+
         protected virtual void InitializeTrackModel ()
         {
         }
@@ -195,7 +219,7 @@ namespace Banshee.Sources
 #region Public Properties
 
         public override int Count {
-            get { return ever_reloaded ? DatabaseTrackModel.UnfilteredCount : SavedCount; }
+            get { return ever_counted ? DatabaseTrackModel.UnfilteredCount : SavedCount; }
         }
 
         public override int FilteredCount {
@@ -248,6 +272,10 @@ namespace Banshee.Sources
             get { return true; }
         }
 
+        public virtual bool Indexable {
+            get { return false; }
+        }
+
         private int saved_count;
         protected int SavedCount {
             get { return saved_count; }
@@ -259,6 +287,19 @@ namespace Banshee.Sources
             return CanAddTracks && source != this;
         }
 
+        public override bool AcceptsUserInputFromSource (Source source)
+        {
+            return base.AcceptsUserInputFromSource (source) && CanAddTracks;
+        }
+                
+        public override bool HasViewableTrackProperties {
+            get { return true; }
+        }
+        
+        public override bool HasEditableTrackProperties {
+            get { return true; }
+        }
+
 #endregion
         
 #region Filters (aka Browsers)
@@ -353,7 +394,7 @@ namespace Banshee.Sources
 
         public virtual void Reload ()
         {
-            ever_reloaded = true;
+            ever_counted = ever_reloaded = true;
             reload_limiter.Execute ();
         }
 
@@ -414,6 +455,9 @@ namespace Banshee.Sources
                 return false;
 
             DatabaseTrackListModel model = (source as ITrackModelSource).TrackModel as DatabaseTrackListModel;
+            if (model == null) {
+                return false;
+            }
             WithTrackSelection (model, AddTrackRange);
             OnTracksAdded ();
             OnUserNotifyUpdated ();
@@ -472,7 +516,7 @@ namespace Banshee.Sources
                 AddSelectedTracks (source);
             }
         }
-
+        
 #endregion
         
 #region Protected Methods
@@ -554,13 +598,7 @@ namespace Banshee.Sources
             }
         }
 
-        protected virtual void AfterInitialized ()
-        {
-            DatabaseTrackModel.Initialize (TrackCache);
-            OnSetupComplete ();
-        }
-
-        private bool ever_reloaded = false;
+        private bool ever_reloaded = false, ever_counted = false;
         public override void Activate ()
         {
             if (!ever_reloaded)
@@ -569,9 +607,9 @@ namespace Banshee.Sources
         
         public override void Deactivate ()
         {
-            DatabaseTrackModel.InvalidateCache ();
+            DatabaseTrackModel.InvalidateCache (false);
             foreach (IFilterListModel filter in AvailableFilters) {
-                filter.InvalidateCache ();
+                filter.InvalidateCache (false);
             }
         }
 
@@ -630,10 +668,10 @@ namespace Banshee.Sources
 
         protected void InvalidateCaches ()
         {
-            track_model.InvalidateCache ();
+            track_model.InvalidateCache (true);
 
             foreach (IFilterListModel filter in CurrentFilters) {
-                filter.InvalidateCache ();
+                filter.InvalidateCache (true);
             }
         }
 
