@@ -41,6 +41,7 @@ namespace Hyena.Data.Sqlite
         private readonly List<VirtualDatabaseColumn> virtual_columns = new List<VirtualDatabaseColumn> ();
         
         private DatabaseColumn key;
+        private int key_select_column_index;
         private HyenaSqliteConnection connection;
         
         private HyenaSqliteCommand create_command;
@@ -82,6 +83,10 @@ namespace Hyena.Data.Sqlite
             get { return "HyenaModelVersions"; }
         }
         
+        public HyenaSqliteConnection Connection {
+            get { return connection; }
+        }
+        
         protected SqliteModelProvider (HyenaSqliteConnection connection)
         {
             this.connection = connection;
@@ -119,6 +124,8 @@ namespace Hyena.Data.Sqlite
                 throw new Exception (String.Format ("The {0} table does not have a primary key", TableName));
             }
             
+            key_select_column_index = select_columns.IndexOf (key);
+            
             CheckVersion ();
             CheckTable ();
         }
@@ -178,7 +185,7 @@ namespace Hyena.Data.Sqlite
                 HyenaTableName, name, version));
         }
         
-        private void CheckTable ()
+        protected void CheckTable ()
         {
             //Console.WriteLine ("In {0} checking for table {1}", this, TableName);
             IDictionary<string, string> schema = connection.GetSchema (TableName);
@@ -206,36 +213,8 @@ namespace Hyena.Data.Sqlite
                 DatabaseColumn c = member is FieldInfo
                     ? new DatabaseColumn ((FieldInfo)member, column)
                     : new DatabaseColumn ((PropertyInfo)member, column);
-                
-                foreach (DatabaseColumn col in columns) {
-                    if (col.Name == c.Name) {
-                        throw new Exception (String.Format (
-                            "{0} has multiple columns named {1}",
-                             TableName, c.Name)
-                        );
-                    }
-                    if (col.Index != null && col.Index == c.Index) {
-                        throw new Exception (String.Format (
-                            "{0} has multiple indecies named {1}",
-                            TableName, c.Name)
-                        );
-                    }
-                }
-                
-                columns.Add (c);
-
-                if (column.Select) {
-                    select_columns.Add (c);
-                }
-                
-                if ((c.Constraints & DatabaseColumnConstraints.PrimaryKey) > 0) {
-                    if (key != null) {
-                        throw new Exception (String.Format (
-                            "Multiple primary keys in the {0} table", TableName)
-                        );
-                    }
-                    key = c;
-                }
+                    
+                AddColumn (c, column.Select);
             }
             VirtualDatabaseColumnAttribute virtual_column = attribute as VirtualDatabaseColumnAttribute;
             if (virtual_column != null) {
@@ -247,6 +226,39 @@ namespace Hyena.Data.Sqlite
             }
         }
         
+        protected void AddColumn (DatabaseColumn c, bool select)
+        {
+            foreach (DatabaseColumn col in columns) {
+                if (col.Name == c.Name) {
+                    throw new Exception (String.Format (
+                        "{0} has multiple columns named {1}",
+                         TableName, c.Name)
+                    );
+                }
+                if (col.Index != null && col.Index == c.Index) {
+                    throw new Exception (String.Format (
+                        "{0} has multiple indecies named {1}",
+                        TableName, c.Name)
+                    );
+                }
+            }
+            
+            columns.Add (c);
+
+            if (select) {
+                select_columns.Add (c);
+            }
+            
+            if ((c.Constraints & DatabaseColumnConstraints.PrimaryKey) > 0) {
+                if (key != null) {
+                    throw new Exception (String.Format (
+                        "Multiple primary keys in the {0} table", TableName)
+                    );
+                }
+                key = c;
+            }
+        }
+        
         protected virtual void CreateTable ()
         {
             connection.Execute (CreateCommand);
@@ -260,12 +272,26 @@ namespace Hyena.Data.Sqlite
             }
         }
         
-        public void Save (T target)
+        protected void CreateIndex (string name, string columns)
         {
-            if (((int)key.GetValue (target)) > 0) {
-                Update (target);
-            } else {
-                key.SetValue (target, Insert (target));
+            Connection.Execute (String.Format (
+                "CREATE INDEX {0} ON {1} ({2})",
+                name, TableName, columns
+            ));
+        }
+        
+        public virtual void Save (T target)
+        {
+            try {
+                if (Convert.ToInt32 (key.GetRawValue (target)) > 0) {
+                    Update (target);
+                } else {
+                    key.SetValue (target, Insert (target));
+                }
+            } catch (Exception e) {
+                Hyena.Log.Exception (e); 
+                Hyena.Log.DebugFormat ("type of key value: {0}", key.GetRawValue (target).GetType ());
+                throw;
             }
         }
         
@@ -308,7 +334,7 @@ namespace Hyena.Data.Sqlite
             connection.Execute (UpdateCommand, GetUpdateParams (target));
         }
         
-        public T Load (IDataReader reader)
+        public virtual T Load (IDataReader reader)
         {
             T item = MakeNewObject ();
             Load (reader, item);
@@ -347,26 +373,28 @@ namespace Hyena.Data.Sqlite
             }
         }
 
-        public T FetchFirstMatching (string condition)
+        public T FetchFirstMatching (string condition, params object [] vals)
         {
-            HyenaSqliteCommand fetch_matching_command = new HyenaSqliteCommand (String.Format ("{0} AND {1}", SelectCommand.Text, condition));
-            using (IDataReader reader = connection.Query (fetch_matching_command)) {
-                if (reader.Read ()) {
-                    return Load (reader);
-                }
+            foreach (T item in FetchAllMatching (String.Format ("{0} LIMIT 1", condition), vals)) {
+                return item;
             }
             return default(T);
         }
         
-        public IEnumerable<T> FetchAllMatching (string condition)
+        public IEnumerable<T> FetchAllMatching (string condition, params object [] vals)
         {
-            HyenaSqliteCommand fetch_matching_command = new HyenaSqliteCommand (String.Format ("{0} AND {1}", SelectCommand.Text, condition));
-            using (IDataReader reader = connection.Query (fetch_matching_command)) {
+            HyenaSqliteCommand fetch_matching_command = CreateFetchCommand (condition);
+            using (IDataReader reader = connection.Query (fetch_matching_command, vals)) {
                 while (reader.Read ()) {
                     yield return Load (reader);
                 }
             }
         }
+
+        public HyenaSqliteCommand CreateFetchCommand (string condition)
+        {
+            return new HyenaSqliteCommand (String.Format ("{0} AND {1}", SelectCommand.Text, condition));
+        }
         
         public IEnumerable<T> FetchRange (int offset, int limit)
         {
@@ -382,7 +410,7 @@ namespace Hyena.Data.Sqlite
             return FetchSingle ((long) id);
         }
         
-        public T FetchSingle (long id)
+        public virtual T FetchSingle (long id)
         {
             using (IDataReader reader = connection.Query (SelectSingleCommand, id)) {
                 if (reader.Read ()) {
@@ -394,12 +422,18 @@ namespace Hyena.Data.Sqlite
         
         protected long PrimaryKeyFor (T item)
         {
-            return (long) key.GetValue (item);
+            return Convert.ToInt64 (key.GetValue (item));
         }
         
-        public void Delete (long id)
+        protected long PrimaryKeyFor (IDataReader reader)
         {
-            connection.Execute (delete_command, id);
+            return Convert.ToInt64 (reader[key_select_column_index]);
+        }
+        
+        public virtual void Delete (long id)
+        {
+            if (id > 0)
+                connection.Execute (DeleteCommand, id);
         }
         
         public void Delete (T item)
@@ -407,7 +441,7 @@ namespace Hyena.Data.Sqlite
             Delete (PrimaryKeyFor (item));
         }
         
-        public void Delete (IEnumerable<T> items)
+        public virtual void Delete (IEnumerable<T> items)
         {
             List<long> ids = new List<long> ();
             long id;
@@ -418,7 +452,7 @@ namespace Hyena.Data.Sqlite
             }
             
             if (ids.Count > 0)
-                connection.Execute (delete_command, ids.ToArray ());
+                connection.Execute (DeleteCommand, ids.ToArray ());
         }
 
         public bool Refresh (T item)
@@ -432,7 +466,7 @@ namespace Hyena.Data.Sqlite
 
             using (IDataReader reader = connection.Query (SelectSingleCommand, id)) {
                 if (reader.Read ()) {
-                    Load (reader);
+                    Load (reader, item);
                     return true;
                 }
             }
@@ -545,7 +579,7 @@ namespace Hyena.Data.Sqlite
         protected virtual HyenaSqliteCommand SelectRangeCommand {
             get {
                 if (select_range_command == null) {
-                    select_command = new HyenaSqliteCommand (
+                    select_range_command = new HyenaSqliteCommand (
                         String.Format (
                             "SELECT {0} FROM {1}{2}{3} LIMIT ?, ?",
                             Select, From,
@@ -619,6 +653,7 @@ namespace Hyena.Data.Sqlite
                 }
                 return primary_key;
             }
+            protected set { primary_key = value; }
         }
         
         private void BuildQuerySql ()
@@ -638,44 +673,42 @@ namespace Hyena.Data.Sqlite
             
             StringBuilder where_builder = new StringBuilder ();
             Dictionary<string, string> tables = new Dictionary<string,string> (virtual_columns.Count + 1);
-            tables.Add (TableName, null);
             bool first_virtual = true;
             foreach (VirtualDatabaseColumn column in virtual_columns) {
-                if (first_virtual) {
-                    first_virtual = false;
-                } else {
-                    where_builder.Append (" AND ");
-                }
                 if (first) {
                     first = false;
                 } else {
                     select_builder.Append (',');
                 }
+
                 select_builder.Append (column.TargetTable);
                 select_builder.Append ('.');
                 select_builder.Append (column.Name);
+
+                bool table_not_joined = !tables.ContainsKey (column.TargetTable);
+                if (first_virtual) {
+                    first_virtual = false;
+                } else if (table_not_joined) {
+                    where_builder.Append (" AND ");
+                }
                 
-                where_builder.Append (column.TargetTable);
-                where_builder.Append ('.');
-                where_builder.Append (column.ForeignKey);
-                where_builder.Append (" = ");
-                where_builder.Append (TableName);
-                where_builder.Append ('.');
-                where_builder.Append (column.LocalKey);
+                if (table_not_joined) {
+                    where_builder.Append (column.TargetTable);
+                    where_builder.Append ('.');
+                    where_builder.Append (column.ForeignKey);
+                    where_builder.Append (" = ");
+                    where_builder.Append (TableName);
+                    where_builder.Append ('.');
+                    where_builder.Append (column.LocalKey);
                 
-                if (!tables.ContainsKey (column.TargetTable)) {
                     tables.Add (column.TargetTable, null);
                 }
             }
             
             StringBuilder from_builder = new StringBuilder ();
-            bool first_tables = true;
+            from_builder.Append (TableName);
             foreach (KeyValuePair<string, string> pair in tables) {
-                if (first_tables) {
-                    first_tables = false;
-                } else {
-                    from_builder.Append (',');
-                }
+                from_builder.Append (',');
                 from_builder.Append (pair.Key);
             }
 
